(* ::Package:: *)

(* ::Input:: *)
(*                   Algorithm: ASPPlusPMinus              *)
(*                       Authors: Ana C. Concei\[CCedilla]\[ATilde]o (aconcei@ualg.pt) & J\[EAcute]ssica Pires(jessicaccpires@gmail.com)          *)
(*                              Year: 2021*)
(*              Institution: Center for Functional Analysis, Linear Structures and Applications (CEAFEL), Universidade do Algarve          *)
(*   Article/Journal: Symbolic computation applied to Cauchy type singular integrals.  Math. Comput. Appl. *)
(*                  *)
(*         *)


         
SetDirectory[NotebookDirectory[]];
Clear[Evaluate[Context[]<>"*"]];
Clear["Global`*"]
(* Options to insert function r(t) *)

DialogInput[Row[{Button["Insert r(t) directly.",DialogReturn[opt=1]],Button["Insert the numerator and poles.",DialogReturn[opt=2]],Button["Insert the zeros and poles.",DialogReturn[opt=3]]}]];

(* Define Operator Subscript[P, +] and Subscript[P, -] *)

PMinus[x_+y_]:=PMinus[x]+PMinus[y]
PMinus[x_-y_]:=PMinus[x]-PMinus[y]
PMinus[a_*x_]:=a *PMinus[x]/;FreeQ[a,t]
PMinus[a_]:=0 /; NumberQ[a]
PMinus[t^n_]:=0 /; n>0
PMinus[t]:=0
PMinus[(t+a_)^n_]:=0 /; Abs[a]>1 && n<0
PMinus[(a_+b_*t)^n_]:=0 /; Abs[a/b]>1 && n<0 

Pplus[x_+y_]:=Pplus[x]+Pplus[y]
Pplus[x_-y_]:=Pplus[x]-Pplus[y]
Pplus[a_ x_]:=a Pplus[x]/;FreeQ[a,t]
Pplus[t^n_]:=0/;n<0
Pplus[t]:=t
Pplus[t^n_]:=t^n/;n>0
Pplus[(t+a_)^n_]:=0/;Abs[a]<1 && n<0
Pplus[(a_+b_ t)^n_]:=0/;Abs[a/b]<1&&n<0
Pplus[(t+a_)^n_]:=(t+a)^n/;Abs[a]>1 && n<0 
Pplus[(a_+b_ t)^n_]:=(a+b t)^n/;Abs[a/b]>1&&n<0
Pplus[a_]:=a /; FreeQ[a,t]

(* Define r(t) *)
Which[opt==1,r[t_]=Factor[Input["Insert r(t):"]];
,
opt==2 ,numerator[t_]=Input["Insert numerator of r(t):"];poles={};
npoles=Input["How many different poles?"];
For[i=0,i< npoles,i++;pole=Input["Input the pole:"];poleM=Input["Input the multiplicity of this pole:"];
AppendTo[poles,{pole,poleM}]];
denominator[t_]=\!\(
\*UnderoverscriptBox[\(\[Product]\), \(i = 1\), \(npoles\)]
\*SuperscriptBox[\((t - poles[\([i, 1]\)])\), \(poles[\([i, 2]\)]\)]\);
r[t_]=Simplify[numerator[t]/denominator[t]]
 ,
opt==3,zeros={};
nzeros=Input["How many different zeros?"];
For[i=0,i< nzeros,i++;zero=Input["Input the zero:"];zeroM=Input["Input the multiplicity of this zero:"];
AppendTo[zeros,{zero,zeroM}];
poles={};
npoles=Input["How many different poles?"];
For[i=0,i< npoles,i++;pole=Input["Input the pole:"];poleM=Input["Input the multiplicity of this pole:"];
AppendTo[poles,{pole,poleM}]];
numerator[t_]=\!\(
\*UnderoverscriptBox[\(\[Product]\), \(i = 1\), \(nzeros\)]
\*SuperscriptBox[\((t - zeros[\([i, 1]\)])\), \(zeros[\([i, 2]\)]\)]\);
denominator[t_]=\!\(
\*UnderoverscriptBox[\(\[Product]\), \(j = 1\), \(npoles\)]
\*SuperscriptBox[\((t - poles[\([j, 1]\)])\), \(poles[\([j, 2]\)]\)]\);
r[t_]=Simplify[numerator[t]/denominator[t]]]
]

poles=Solve[Denominator[r[t]]==0,t];

<<"APoles.m";


 If[circlep!=0,Print["r \[NotElement] R(\[DoubleStruckCapitalT])"],


(* Decompose r(t) *)
decR=Function[{},c=Coefficient[Denominator[r[t]],t,Exponent[Denominator[r[t]],t]];
poles=Solve[Denominator[r[t]]==0,t];
polesM=Tally[poles[[All,1,2]]];
npolesM=Length[polesM];
(*For[i=0,i<npolesM,i++;If[Head[polesM[[i,1]]]\[Equal]Root&&Abs[polesM[[i,1]]]<1,polesM[[i,1]]=Subscript[\[Alpha], i]]];
For[i=0,i<npolesM,i++;If[Head[polesM[[i,1]]]\[Equal]Root&&Abs[polesM[[i,1]]]>1,polesM[[i,1]]=Subscript[\[Beta], i]]];*)

den[t_]=c*\!\(
\*UnderoverscriptBox[\(\[Product]\), \(i = 1\), \(npolesM\)]
\*SuperscriptBox[\((t - polesM[\([i, 1]\)])\), \(polesM[\([i, 2]\)]\)]\);
rDec[t_]=Numerator[r[t]]/den[t];

Apart[rDec[t],t]

];

decomR[t_]=If[opt==1,decR[],Apart[r[t]]];
R[t_]=Expand[decomR[t]];

(* Usando o PPlus *)
PPlusR[t_]=Pplus[R[t]];
PMinusR[t_]=Simplify[R[t]-PPlusR[t],TimeConstraint->Infinity];
Sr[t_]=Simplify[2*PPlusR[t]-R[t],TimeConstraint->Infinity];



(* Usando o PMinus 
PMinusR[t_]=PMinus[R[t]];
PPlusR[t_]=Simplify[R[t]-PMinusR[t]];

Sr[t_]=Simplify[2*PPlusR[t]-R[t]];
*)

Print[
"\!\(\*
StyleBox[\"r\", \"Section\"]\)\!\(\*
StyleBox[\"(\", \"Section\"]\)\!\(\*
StyleBox[\"t\", \"Section\"]\)\!\(\*
StyleBox[\")\", \"Section\"]\)\!\(\*
StyleBox[\" \", \"Section\"]\)\!\(\*
StyleBox[\"=\", \"Section\"]\)\!\(\*
StyleBox[\" \", \"Section\"]\)",r[t]];
Print[
"\!\(\*
StyleBox[SubscriptBox[\"S\", \"\[DoubleStruckCapitalT]\"], \"Section\"]\)\!\(\*
StyleBox[\"r\", \"Section\"]\)\!\(\*
StyleBox[\"(\", \"Section\"]\)\!\(\*
StyleBox[\"t\", \"Section\"]\)\!\(\*
StyleBox[\")\", \"Section\"]\)\!\(\*
StyleBox[\" \", \"Section\"]\)\!\(\*
StyleBox[\"=\", \"Section\"]\)\!\(\*
StyleBox[\" \", \"Section\"]\)",Sr[t]];
Print[
"\!\(\*
StyleBox[SubscriptBox[\"P\", \"+\"], \"Section\"]\)\!\(\*
StyleBox[\"r\", \"Section\"]\)\!\(\*
StyleBox[\"(\", \"Section\"]\)\!\(\*
StyleBox[\"t\", \"Section\"]\)\!\(\*
StyleBox[\")\", \"Section\"]\)\!\(\*
StyleBox[\" \", \"Section\"]\)\!\(\*
StyleBox[\"=\", \"Section\"]\)\!\(\*
StyleBox[\" \", \"Section\"]\)",PPlusR[t]];
Print[
"\!\(\*
StyleBox[SubscriptBox[\"P\", \"-\"], \"Section\"]\)\!\(\*
StyleBox[\"r\", \"Section\"]\)\!\(\*
StyleBox[\"(\", \"Section\"]\)\!\(\*
StyleBox[\"t\", \"Section\"]\)\!\(\*
StyleBox[\")\", \"Section\"]\)\!\(\*
StyleBox[\" \", \"Section\"]\)\!\(\*
StyleBox[\"=\", \"Section\"]\)\!\(\*
StyleBox[\" \", \"Section\"]\)",PMinusR[t]];
]



(* ::Input:: *)
(*r=1/(Root[1 - 6 # + 5 #^4 + #^6& , 1, 0]-Root[1 - 6 # + 5 #^4 + #^6& , 2, 0]);*)
(*ReplacePart[r,Root[1 - 6 # + 5 #^4 + #^6& , 1, 0]->Subscript[\[Alpha], 1]]*)
(*r*)
