(* ::Package:: *)

(* ::Input:: *)
(*                   Algorithm: SInt_ 2.0              *)
(*                       Authors: Ana C. Concei\[CCedilla]\[ATilde]o (aconcei@ualg.pt) & J\[EAcute]ssica Pires(jessicaccpires@gmail.com)          *)
(*                              Year: 2021*)
(*              Institution: Center for Functional Analysis, Linear Structures and Applications (CEAFEL), Universidade do Algarve          *)
(*   Article/Journal: Symbolic computation applied to Cauchy type singular integrals.  Math. Comput. Appl. *)
(*                  *)
(*         *)


(* ::Input::Initialization:: *)
Clear["Global`*"]
SetDirectory[NotebookDirectory[]];
Clear[Evaluate[Context[]<>"*"]];
DialogInput[Row[{Button["Insert r(t) directly.",DialogReturn[opt=1]],Button["Insert the numerator and poles.",DialogReturn[opt=2]],Button["Insert the zeros and poles.",DialogReturn[opt=3]]}]];
DialogInput[Row[{Button["Use general \!\(\*SubscriptBox[\(x\), \(+\)]\)(t).",DialogReturn[optX=1]],Button["Use particular \!\(\*SubscriptBox[\(x\), \(+\)]\)(t).",DialogReturn[optX=2]]}]];
DialogInput[Row[{Button["Use general \!\(\*SubscriptBox[\(y\), \(-\)]\)(t).",DialogReturn[optY=1]],Button["Use particular \!\(\*SubscriptBox[\(y\), \(-\)]\)(t).",DialogReturn[optY=2]]}]];

ruleF={FractionBox[x_,y_]->x/y};

Clear[Pplus];
Pplus[x_+y_]:=Pplus[x]+Pplus[y];
Pplus[a_ x_]:=a Pplus[x]/;FreeQ[a,t];
Pplus[t^n_]:=0/;n<0;
Pplus[t^n_]:=t^n/;n>0;
Pplus[t]:=t;
Pplus[(t+a_)^n_]:=0/;Abs[a]<1&&n<0;
Pplus[(a_+b_ t)^n_]:=0/;Abs[a/b]<1&&n<0;
Pplus[(t+a_)^n_]:=(t+a)^n/;Abs[a]>1 && n<0 
Pplus[(a_+b_ t)^n_]:=(a+b t)^n/;Abs[a/b]>1&&n<0
Pplus[a_]:=a /; FreeQ[a,t]
Pplus[Xplus[t]*t^n_]:=Xplus[t]*t^n-\!\(
\*UnderoverscriptBox[\(\[Sum]\), \(i = 1\), \(-n\)]\((
\*FractionBox[\((D[Xplus[t], {t, i - 1}] /. t -> 0)\), \(\((i - 1)\)!\)] 
\*SuperscriptBox[\(t\), \(i + n - 1\)])\)\)/;n<0;
Pplus[Xplus[t]*t^n_]:=Xplus[t]*t^n/;n>0;
Pplus[Xplus[t]]:=Xplus[t];
Pplus[Xplus[t]*(t+a_)^n_]:=Xplus[t]*(t+a)^n-\!\(
\*UnderoverscriptBox[\(\[Sum]\), \(i = 1\), \(-n\)]\((
\*FractionBox[\((D[Xplus[t], {t, i - 1}] /. t -> \(-a\))\), \(\((i - 1)\)!\)] 
\*SuperscriptBox[\((t + a)\), \(i + n - 1\)])\)\)/;Abs[a]<1 && n<0;
Pplus[Xplus[t]*(t+a_)^n_]:=Xplus[t]*(t+a)^n/;Abs[a]>1 && n<0;
Pplus[Xplus[t]*(t+a_)^n_]:=Xplus[t]*(t+a)^n/;n>0;
Pplus[Xplus[t]*(t+a_)]:=Xplus[t]*(t+a);
Pplus[Xplus[t]*(a_+b_ t)^n_]:=b^n*Pplus[Xplus[t]*(t+a/b)^n];
Pplus[Xplus[t]*t]:=Xplus[t]*t;

Pplus[Yminus[t]]:=Conjugate[Yplus[0]];
Pplus[Yminus[t]*t^n_]:=0/;n<0;
Pplus[Yminus[t]*t^n_.]:=\!\(
\*UnderoverscriptBox[\(\[Sum]\), \(i = 0\), \(n\)]\(
\*FractionBox[\(Conjugate[D[Yplus[t], {t, i}] /. t -> 0]\), \(i!\)] 
\*SuperscriptBox[\(t\), \(n - i\)]\)\)/;n>0; 
Pplus[Yminus[t]*t]:=\!\(
\*UnderoverscriptBox[\(\[Sum]\), \(i = 0\), \(1\)]\(
\*FractionBox[\(Conjugate[D[Yplus[t], {t, i}] /. t -> 0]\), \(i!\)] 
\*SuperscriptBox[\(t\), \(1 - i\)]\)\) ;
Pplus[Yminus[t]*(t+a_)^n_]:=0/;Abs[a]<1&&n<0;
Pplus[Yminus[t]*(t+a_)^n_]:=\!\(
\*UnderoverscriptBox[\(\[Sum]\), \(i = 1\), \(-n\)]\((
\*FractionBox[\((D[Yminus[t], {t, i - 1}] /. t -> \(-a\))\), \(\((i - 1)\)!\)] 
\*SuperscriptBox[\((t + a)\), \(i + n - 1\)])\)\)/;Abs[a]>1&&n<0;
Pplus[Yminus[t]*(a_+b_ t)^n_]:=b^n*Pplus[Yminus[t]*(t+a/b)^n];



funcSInt=Function[{},in\[IAcute]cio0=AbsoluteTime[];

zeros=Solve[Numerator[r[t]]==0];
poles=Solve[Denominator[r[t]]==0];
<<"AZeros.m";
<< "APoles.m";

If[circlep!=0,Print["r \[NotElement] R(\[DoubleStruckCapitalT])"],LeadingCoeff[poly_,t_]:=Coefficient[poly,t,Exponent[poly,t]];

decompR=Function[{},den[t_]=Denominator[r[t]];
				      a=LeadingCoeff[den[t],t];
				      solP=Solve[den[t]==0,t];(* List of rules: t \[Rule] pole *)
				      p=Tally[solP[[All,1,2]]]; (* List of poles and multiplicities *)
				      n=Length@p;
				      denominator[t_]=a*\!\(
\*UnderoverscriptBox[\(\[Product]\), \(j = 1\), \(n\)]
\*SuperscriptBox[\((t - p[\([j, 1]\)])\), \(p[\([j, 2]\)]\)]\);
				      RFact[t_]=Numerator[r[t]]/denominator[t];
				      Apart[RFact[t],t]];
decR[t_]=If[opt==1,decompR[]
		   (*else*),Apart[r[t],t]];
X[t_]=Expand[decR[t]*Xplus[t]];
Y[t_]=Expand[decR[t]*Yminus[t]];
PplusX[t_]=Simplify[Pplus[X[t]]/.ruleF];
PplusY[t_]=Simplify[Pplus[Y[t]]/.ruleF];



printFormat={Xplus->SubPlus[x],Yplus->SubPlus[y], Yminus->SubMinus[y],Conjugate[aa_]->
\!\(\*OverscriptBox[\(aa\), \(_\)]\)};

SX[t_]=Simplify[2*PplusX[t]-X[t]/.ruleF];
SY[t_]=Simplify[2*PplusY[t]-Y[t]/.ruleF];

Print[""];
Print["\!\(\*
StyleBox[\"Input\",\nFontSize->24,\nFontColor->RGBColor[0.6, 0.4, 0.2]]\)\!\(\*
StyleBox[\" \",\nFontSize->24,\nFontColor->RGBColor[0.6, 0.4, 0.2]]\)\!\(\*
StyleBox[\"Options\",\nFontSize->24,\nFontColor->RGBColor[0.6, 0.4, 0.2]]\)\!\(\*
StyleBox[\":\",\nFontSize->24,\nFontColor->RGBColor[0.6, 0.4, 0.2]]\)"];
Print["\!\(\*
StyleBox[\"r\",\nFontSize->24]\)\!\(\*
StyleBox[\"(\",\nFontSize->24]\)\!\(\*
StyleBox[\"t\",\nFontSize->24]\)\!\(\*
StyleBox[\")\",\nFontSize->24]\)\!\(\*
StyleBox[\" \",\nFontSize->24]\)\!\(\*
StyleBox[\"=\",\nFontSize->24]\)\!\(\*
StyleBox[\" \",\nFontSize->24]\)", Style[r[t],22],"\!\(\*
StyleBox[\";\",\nFontSize->24]\)\!\(\*
StyleBox[\"    \",\nFontSize->24]\)\!\(\*SubscriptBox[
StyleBox[\"x\",\nFontSize->24], 
StyleBox[\"+\",\nFontSize->18]]\)\!\(\*
StyleBox[\"(\",\nFontSize->18]\)\!\(\*
StyleBox[\"t\",\nFontSize->18]\)\!\(\*
StyleBox[\")\",\nFontSize->18]\)\!\(\*
StyleBox[\" \",\nFontSize->18]\)\!\(\*
StyleBox[\"=\",\nFontSize->24]\)\!\(\*
StyleBox[\" \",\nFontSize->16]\)", Style[Xplus[t]/.printFormat,22],
									 "\!\(\*
StyleBox[\";\",\nFontSize->24]\)\!\(\*
StyleBox[\"    \",\nFontSize->24]\)\!\(\*SubscriptBox[
StyleBox[\"y\",\nFontSize->24], 
StyleBox[\"-\",\nFontSize->18]]\)\!\(\*
StyleBox[\"(\",\nFontSize->18]\)\!\(\*
StyleBox[\"t\",\nFontSize->18]\)\!\(\*
StyleBox[\")\",\nFontSize->18]\)\!\(\*
StyleBox[\" \",\nFontSize->18]\)\!\(\*
StyleBox[\"=\",\nFontSize->24]\)\!\(\*
StyleBox[\" \",\nFontSize->16]\)", Style[Yminus[t]/.printFormat,22]];
Print[""];
Print["\!\(\*
StyleBox[\"X\",\nFontSize->24]\)\!\(\*
StyleBox[\"(\",\nFontSize->24]\)\!\(\*
StyleBox[\"t\",\nFontSize->24]\)\!\(\*
StyleBox[\")\",\nFontSize->24]\)\!\(\*
StyleBox[\" \",\nFontSize->24]\)\!\(\*
StyleBox[\"=\",\nFontSize->24]\)\!\(\*
StyleBox[\" \",\nFontSize->24]\)\!\(\*
StyleBox[\"r\",\nFontSize->24]\)\!\(\*
StyleBox[\"(\",\nFontSize->24]\)\!\(\*
StyleBox[\"t\",\nFontSize->24]\)\!\(\*
StyleBox[\")\",\nFontSize->24]\)\!\(\*SubscriptBox[
StyleBox[\"x\",\nFontSize->24], 
StyleBox[\"+\",\nFontSize->18]]\)\!\(\*
StyleBox[\"(\",\nFontSize->18]\)\!\(\*
StyleBox[\"t\",\nFontSize->18]\)\!\(\*
StyleBox[\")\",\nFontSize->18]\)\!\(\*
StyleBox[\" \",\nFontSize->18]\)\!\(\*
StyleBox[\"=\",\nFontSize->24]\)\!\(\*
StyleBox[\" \",\nFontSize->16]\)\!\(\*
StyleBox[\" \",\nFontSize->16]\)\!\(\*
StyleBox[\"=\",\nFontSize->16]\)\!\(\*
StyleBox[\" \",\nFontSize->16]\)",Style[X[t]//.printFormat,22]];
Print[""];
Print["\!\(\*
StyleBox[\"Y\",\nFontSize->24]\)\!\(\*
StyleBox[\"(\",\nFontSize->24]\)\!\(\*
StyleBox[\"t\",\nFontSize->24]\)\!\(\*
StyleBox[\")\",\nFontSize->24]\)\!\(\*
StyleBox[\" \",\nFontSize->24]\)\!\(\*
StyleBox[\"=\",\nFontSize->24]\)\!\(\*
StyleBox[\" \",\nFontSize->24]\)\!\(\*
StyleBox[\"r\",\nFontSize->24]\)\!\(\*
StyleBox[\"(\",\nFontSize->24]\)\!\(\*
StyleBox[\"t\",\nFontSize->24]\)\!\(\*
StyleBox[\")\",\nFontSize->24]\)\!\(\*SubscriptBox[
StyleBox[\"y\",\nFontSize->24], 
StyleBox[\"-\",\nFontSize->18]]\)\!\(\*
StyleBox[\"(\",\nFontSize->18]\)\!\(\*
StyleBox[\"t\",\nFontSize->18]\)\!\(\*
StyleBox[\")\",\nFontSize->18]\)\!\(\*
StyleBox[\" \",\nFontSize->16]\)\!\(\*
StyleBox[\"=\",\nFontSize->16]\)\!\(\*
StyleBox[\" \",\nFontSize->16]\)",Style[Y[t]//.printFormat,22]];
Print[""];
Print["\!\(\*
StyleBox[\"Singular\",\nFontSize->36,\nFontColor->RGBColor[1, 0.5, 0]]\)\!\(\*
StyleBox[\" \",\nFontSize->36,\nFontColor->RGBColor[1, 0.5, 0]]\)\!\(\*
StyleBox[\"Integrals\",\nFontSize->36,\nFontColor->RGBColor[1, 0.5, 0]]\)\!\(\*
StyleBox[\":\",\nFontSize->36,\nFontColor->RGBColor[1, 0.5, 0]]\)\!\(\*
StyleBox[\" \",\nFontSize->36,\nFontColor->RGBColor[1, 0.5, 0]]\)"];
Print[""];
Print["\!\(\*
StyleBox[SubscriptBox[\"S\", \"\[DoubleStruckCapitalT]\"],\nFontSize->36]\)\!\(\*
StyleBox[\"X\",\nFontSize->36]\)\!\(\*
StyleBox[\"(\",\nFontSize->36]\)\!\(\*
StyleBox[\"t\",\nFontSize->36]\)\!\(\*
StyleBox[\")\",\nFontSize->36]\)\!\(\*
StyleBox[\" \",\nFontSize->36]\)\!\(\*
StyleBox[\"=\",\nFontSize->36]\)\!\(\*
StyleBox[\" \",\nFontSize->36]\)", Style[SX[t]//.printFormat,22]];
Print[""];
Print["\!\(\*
StyleBox[SubscriptBox[\"S\", \"\[DoubleStruckCapitalT]\"],\nFontSize->36]\)\!\(\*
StyleBox[\"Y\",\nFontSize->36]\)\!\(\*
StyleBox[\"(\",\nFontSize->36]\)\!\(\*
StyleBox[\"t\",\nFontSize->36]\)\!\(\*
StyleBox[\")\",\nFontSize->36]\)\!\(\*
StyleBox[\" \",\nFontSize->36]\)\!\(\*
StyleBox[\"=\",\nFontSize->36]\)\!\(\*
StyleBox[\" \",\nFontSize->36]\)", Style[SY[t]//.printFormat,22]];
Print[""];
fim0=AbsoluteTime[];
Print["\!\(\*
StyleBox[\"TOTAL\",\nFontSize->18,\nFontColor->RGBColor[1, 0.5, 0]]\)\!\(\*
StyleBox[\" \",\nFontSize->18,\nFontColor->RGBColor[1, 0.5, 0]]\)\!\(\*
StyleBox[\"EXECUTION\",\nFontSize->18,\nFontColor->RGBColor[1, 0.5, 0]]\)\!\(\*
StyleBox[\" \",\nFontSize->18,\nFontColor->RGBColor[1, 0.5, 0]]\)\!\(\*
StyleBox[\"TIME\",\nFontSize->18,\nFontColor->RGBColor[1, 0.5, 0]]\)\!\(\*
StyleBox[\" \",\nFontSize->18]\)\!\(\*
StyleBox[\"=\",\nFontSize->18]\)\!\(\*
StyleBox[\" \",\nFontSize->18]\)",fim0-in\[IAcute]cio0, " SECONDS"];
Print[""];]];




Which[opt==1,r[t_]=Input["Insert r(t):"];Print["r(t)=",r[t]]
,
opt==2 ,numerator[t_]=Input["Insert numerator of r(t):"];
zeros=Solve[numerator[t]==0];
poles={};
npoles=Input["How many different poles?"];
For[i=0,i< npoles,i++;pole=Input["Input the pole:"];poleM=Input["Input the multiplicity of this pole:"];
AppendTo[poles,{pole,poleM}]];
denominator[t_]=\!\(
\*UnderoverscriptBox[\(\[Product]\), \(i = 1\), \(npoles\)]
\*SuperscriptBox[\((t - poles[\([i, 1]\)])\), \(poles[\([i, 2]\)]\)]\);
r[t_]=Simplify[numerator[t]/denominator[t]];Print["r(t)=",r[t]]
 ,
opt==3,zeros={};
nzeros=Input["How many different zeros?"];
For[i=0,i< nzeros,i++;zero=Input["Input the zero:"];zeroM=Input["Input the multiplicity of this zero:"];
AppendTo[zeros,{zero,zeroM}];
poles={};
npoles=Input["How many different poles?"];
For[i=0,i< npoles,i++;pole=Input["Input the pole:"];poleM=Input["Input the multiplicity of this pole:"];
AppendTo[poles,{pole,poleM}]]];
numerator[t_]=\!\(
\*UnderoverscriptBox[\(\[Product]\), \(i = 1\), \(nzeros\)]
\*SuperscriptBox[\((t - zeros[\([i, 1]\)])\), \(zeros[\([i, 2]\)]\)]\);
denominator[t_]=\!\(
\*UnderoverscriptBox[\(\[Product]\), \(j = 1\), \(npoles\)]
\*SuperscriptBox[\((t - poles[\([j, 1]\)])\), \(poles[\([j, 2]\)]\)]\);
r[t_]=Simplify[numerator[t]/denominator[t]];Print["r(t)=",r[t]]]
If[optX==2,Xplus[t_]=Input["Please insert x+(t):"];poles=Solve[Denominator[Xplus[t]]==0];
<<"APoles.m";

If[circlep!=0,Print["x+ is not in R(\[DoubleStruckCapitalT])"],
If[diskp!=0,Print["x+ is not in R+(\[DoubleStruckCapitalT])"],
If[optY==2,Yminus[t_]=Input["Please insert y-(t):"];
		  Yplus[t_]=Simplify[ Conjugate[Yminus[t]]/.Conjugate[t]->1/t];poles=Solve[Denominator[Yplus[t]]==0];Clear[circlep];Clear[diskp];
<<"APoles.m";
If[circlep!=0,Print["y+ is not in R"],If[diskp!=0,Print["y+ is not in R+"],funcSInt[]]],funcSInt[]]]],
If[optY==2,Yminus[t_]=Input["Please insert y-(t):"];
		  Yplus[t_]=Simplify[ Conjugate[Yminus[t]]/.Conjugate[t]->1/t];poles=Solve[Denominator[Yplus[t]]==0];Clear[circlep];Clear[diskp];
<<"APoles.m";
If[circlep!=0,Print["y+ is not in R"],If[diskp!=0,Print["y+ is not in R+"],funcSInt[]]],funcSInt[]]]




